/* 
   Self-adjusting clock for CET time zone with DST 
   by kk99 
   2018 
*/

#include <SoftwareSerial.h>
#include <TinyGPS++.h>
#include <SPI.h>
#include <U8g2lib.h>
#include <Timezone.h>
#include <Arduino.h>

// GPS handle
TinyGPSPlus gps;

// EDP handle
U8G2_IL3820_V2_296X128_1_4W_HW_SPI u8g2(U8G2_R0, /* cs=*/ 10, /* dc=*/ 9, /* reset=*/ 8);

// Central European Time
TimeChangeRule CEST = {"CEST", Last, Sun, Mar, 2, 120}; // Central European Summer Time
TimeChangeRule CET = {"CET ", Last, Sun, Oct, 3, 60};   // Central European Standard Time
Timezone CE(CEST, CET);
TimeChangeRule *tcr;

// Serial handle
SoftwareSerial softSerial(3, 2);

void setup() {
  // put your setup code here, to run once:
  u8g2.begin();
  softSerial.begin(9600);
  setTime(00, 00, 00, 01, 01, 1970);
}

void loop() {
  // put your main code here, to run repeatedly:
  readGPSData(1000);
  updateTime();
  displayTime();
  delay(59000);
}

static void readGPSData(unsigned long timeoutMs)
{
  unsigned long start = millis();
  do
  {
    while (softSerial.available())
      gps.encode(softSerial.read());
  } while (millis() - start < timeoutMs);
}

static void updateTime(void) {
  if (gps.time.isValid() && gps.date.isValid()) {
    setTime(gps.time.hour(), gps.time.minute(), gps.time.second(), gps.date.day(), gps.date.month(), gps.date.year());
  }
}

static void displayTime(void)
{
  const unsigned timeLength = 6;
  char timeValue[timeLength];
  time_t utc = now();
  time_t local = CE.toLocal(utc, &tcr);
  snprintf(timeValue, timeLength, "%02d:%02d", hour(local), minute(local));
  u8g2.firstPage();
  do {
    u8g2.setFont(u8g2_font_logisoso78_tn);
    u8g2.drawStr(26, 103, timeValue);
    } while (u8g2.nextPage());
}
